@@ -578,7 +578,16 @@ class Sf:
         batches = len(record_dataframe) / batchsize
         result_list = await asyncio.gather(*[async_delete(object_api= object_api, record_dataframe=batch) for batch in np.array_split(record_dataframe, batches)])
         return pd.concat(result_list).reset_index(drop=True)
-
+    
+    def entity_access_query(self, entity_type):
+        # Retrieves information about which Profile or PermissionSet
+        # grants permission to which Setup Entity (i.e. ApexPage, ApexClass, TabSets, etc)
+        
+        entity_access = transform_sf_result_set_rec(self.sf.query_all(f"""
+        SELECT Id, Parent.Id, Parent.ProfileId, Parent.Profile.Name, Parent.Name,SetupEntityId,SetupEntityType FROM
+        SetupEntityAccess WHERE SetupEntityType = '{entity_type}'""")['records'])
+        # entity_access.drop(columns=['PermissionSet.Profile'], inplace=True)
+        return {entity_type: entity_access}
     ### ----------------------------------------------------------------------------------------------------
     ### Asynchronous Metadata Functions
     ### ----------------------------------------------------------------------------------------------------
@@ -666,6 +675,44 @@ class Sf:
         
         return result_dict
     
+    
+    async def entity_access_bulk_query(self, entity_types: list):
+        async_entity_access_query = async_wrap(self.entity_access_query)
+        results = await asyncio.gather(*[async_entity_access_query(entity_type) for entity_type in entity_types])
+        merged_dict = {}
+        for result in results:
+            for key, value in result.items():
+                merged_dict[key] = value
+        return merged_dict
+
+
+    async def apex_pages_profiles_and_permission_set_access_query(self, apex_pages_to_check: list[str]):
+        query_results = await self.entity_access_bulk_query(['ApexPage'])
+
+        entity_access_apex_pages_async =  query_results['ApexPage']
+
+        entity_permission_set_access_apex_pages_async = entity_access_apex_pages_async[entity_access_apex_pages_async['Profile.Name'].isna()]
+        entity_profile_access_apex_pages_async = entity_access_apex_pages_async[entity_access_apex_pages_async['Profile.Name'].notna()]
+
+        # filter = ['Scheduler_Administration_vod','Network_Admin_Page_vod','searchAccts_vod']
+        filter_join = "','".join(apex_pages_to_check)
+        where_clause = f" WHERE Name IN ('{filter_join}')"
+        apex_pages_query = transform_sf_result_set_rec(self.sf.query_all(f"""
+            SELECT Id, Name From ApexPage{where_clause}""")['records'])
+
+        permission_sets_with_apex_page_access = entity_permission_set_access_apex_pages_async[entity_permission_set_access_apex_pages_async['SetupEntityAccess.SetupEntityId'].isin(apex_pages_query['ApexPage.Id'].unique())]
+        profiles_with_apex_page_access = entity_profile_access_apex_pages_async[entity_profile_access_apex_pages_async['SetupEntityAccess.SetupEntityId'].isin(apex_pages_query['ApexPage.Id'].unique())]
+        permission_sets_with_apex_page_access = pd.merge(permission_sets_with_apex_page_access, apex_pages_query, left_on='SetupEntityAccess.SetupEntityId', right_on='ApexPage.Id', how='inner').drop(columns=['ApexPage.Id'], axis=1)
+        profiles_with_apex_page_access = pd.merge(profiles_with_apex_page_access, apex_pages_query, left_on='SetupEntityAccess.SetupEntityId', right_on='ApexPage.Id', how='inner').drop(columns=['ApexPage.Id'], axis=1)
+
+        profiles_with_apex_page_access = profiles_with_apex_page_access[['PermissionSet.ProfileId','Profile.Name','SetupEntityAccess.SetupEntityType','ApexPage.Name']]
+        profiles_with_apex_page_access.columns = ['Profile Id', 'Profile Name', 'Setup Entity Type', 'Visualforce (Apex) Name']
+        
+        permission_sets_with_apex_page_access = permission_sets_with_apex_page_access[['PermissionSet.Id','PermissionSet.Name','SetupEntityAccess.SetupEntityType','ApexPage.Name']]
+        permission_sets_with_apex_page_access.columns = ['PermissionSet Id', 'PermissionSet Name', 'Setup Entity Type', 'Visualforce (ApexPage) Name']
+        
+        return {'profiles_with_apex_page_access': profiles_with_apex_page_access, 'permission_sets_with_apex_page_access': permission_sets_with_apex_page_access}
+
     ### ----------------------------------------------------------------------------------------------------
     ### Synchronous Metadata Functions
     ### ----------------------------------------------------------------------------------------------------
